package com.maileon.api.blacklists;

import com.maileon.api.XmlUtils;
import com.maileon.api.utils.MimeTypes;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;

import javax.ws.rs.Consumes;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.ext.MessageBodyReader;
import javax.ws.rs.ext.MessageBodyWriter;
import javax.ws.rs.ext.Provider;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

@Provider
@Consumes(MimeTypes.MAILEON_API_XML)
@Produces(MimeTypes.MAILEON_API_XML)
public class AddEntriesActionXmlSerializer implements MessageBodyReader<AddEntriesAction>, MessageBodyWriter<AddEntriesAction> {

    private static final String ACTION_ELEMENT_NAME = "add_entries_action";
    private static final String NAME_ELEMENT_NAME = "import_name";
    private static final String ENTRIES_ELEMENT_NAME = "entries";
    private static final String ENTRY_ELEMENT_NAME = "entry";

    @Override
    public boolean isReadable(Class<?> type, Type type1, Annotation[] antns, MediaType mt) {
        return type1 == AddEntriesAction.class;
    }

    @Override
    public AddEntriesAction readFrom(Class<AddEntriesAction> type, Type type1, Annotation[] antns,
            MediaType mt, MultivaluedMap<String, String> mm, InputStream in) throws IOException,
            WebApplicationException {
        try {
            Element filters = XmlUtils.parseXmlDocument(in);
            return deserialize(filters);
        } catch (Throwable e) {
            throw new WebApplicationException(e);
        }
    }

    private static AddEntriesAction deserialize(Element actionElement) {
        AddEntriesAction result = new AddEntriesAction();
        if (ACTION_ELEMENT_NAME.equals(actionElement.getName())) {
            Element name = actionElement.element(NAME_ELEMENT_NAME);
            result.setImportName(name.getText());
            List<String> entries = new ArrayList<>();
            Element entriesElement = actionElement.element(ENTRIES_ELEMENT_NAME);
            for (@SuppressWarnings("rawtypes") Iterator iterator = entriesElement.elementIterator(); iterator.hasNext();) {
                Element entryElement = (Element) iterator.next();
                if (ENTRY_ELEMENT_NAME.equals(entryElement.getName())) {
                    entries.add(entryElement.getText());
                }
            }
            result.setEntries(entries);
        }
        return result;
    }

    @Override
    public boolean isWriteable(Class<?> type, Type type1, Annotation[] antns, MediaType mt) {
        return type1 == AddEntriesAction.class;
    }

    @Override
    public void writeTo(AddEntriesAction t, Class<?> type, Type type1, Annotation[] antns, MediaType mt,
            MultivaluedMap<String, Object> mm, OutputStream out) throws IOException, WebApplicationException {
        if (t == null) {
            throw new WebApplicationException();
        }

        Element root;
        try {
            root = serialize(t);
            XMLWriter writer = new XMLWriter(out, OutputFormat.createPrettyPrint());
            writer.write(root);
            writer.close();
        } catch (Exception e) {
            throw new WebApplicationException(e);
        }

    }

    public static Element serialize(AddEntriesAction action) {
        Element blacklistElement = DocumentHelper.createElement(ACTION_ELEMENT_NAME);

        if (action.getImportName() != null) {
            XmlUtils.appendChildElementSafe(blacklistElement, NAME_ELEMENT_NAME).setText(action.getImportName());
        }

        if (action.getEntries() != null) {
            Element entriesElement = XmlUtils.appendChildElementSafe(blacklistElement, ENTRIES_ELEMENT_NAME);
            for (String entry: action.getEntries()) {
                Element entryElement = DocumentHelper.createElement(ENTRY_ELEMENT_NAME);
                entryElement.setText(entry);
                entriesElement.add(entryElement);
            }
        }

       return blacklistElement;
    }

    @Override
    public long getSize(AddEntriesAction t, Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
        return -1;
    }

}
