package com.maileon.api.ping;
import com.maileon.api.MaileonConfiguration;
import com.maileon.api.MaileonException;

/**
 * Test driver for the ping service
 *
 */
public class MaileonPingUtility {

	// For our current development setup
	private static final String DEFAULT_API_URL = "http://localhost:8080/de.xqueue.eagle.api";
	private static final String DEFAULT_API_KEY = "91a25687-3fa0-41c2-8982-8bfcb2b3500a";

	private static final boolean DEVELOPER_MODE = true;

	/**
	 * The {@link MaileonConfiguration} that will be used for the API calls
	 */
	private static MaileonConfiguration config = null;


	public static void main(String[] args) {
		if (args.length == 2) {
			// NOTE: extension point: validate input: URL + UUID?
			config = new MaileonConfiguration(args[0], args[1]);
		} else {
			System.out.println("Usage: MaileonPingUtil API-URL API-KEY");
			System.out.println("Sample: MaileonPingUtil http://localhost:8080/de.xqueue.eagle.api 91a25687-3fa0-41c2-8982-8bfcb2b3500a");

			// If developing use predefined URL and KEY
			if (DEVELOPER_MODE) {
				System.out.println("Running in developer mode, using defauls...");
				config = new MaileonConfiguration(DEFAULT_API_URL, DEFAULT_API_KEY);
			}
			else return;
		}

		System.out.println(String.format("Running API tests on %s for API-Key: %s", config.getBaseUri(), config.getApiKey()));

		MaileonPingService mps = new MaileonPingService(config);
		mps.setDebug(false); // Enable this for more debugging information

		checkRetrieve(mps);
		checkCreate(mps);
		checkUpdate(mps);
		checkDelete(mps);
	}


	/**
	 * Check if the API-Key has retrieve rights
	 * @param mps The {@link MaileonPingService} to use
	 */
	public static void checkRetrieve(MaileonPingService mps) {
		System.out.print(" - Testing 'retrieve': ");
		try {
			mps.checkRetrieve();
			System.out.println("OK");
		} catch (MaileonException e) {
			System.out.println(String.format("FAILED [%s - %s (%s)]", e.getStatusCode(), e.getReasonPhrase(), e.getMessage()));
		} catch (Exception e) {
			// Different handling; e.g. when returning null a nullpointer is thrown.
			System.out.println(e.getMessage());
		}
	}


	/**
	 * Check if the API-Key has create rights
	 * @param mps The {@link MaileonPingService} to use
	 */
	public static void checkCreate(MaileonPingService mps) {
		System.out.print(" - Testing 'create': ");
		try {
			mps.checkCreate();
			System.out.println("OK");
		} catch (MaileonException e) {
			System.out.println(String.format("FAILED [%s - %s (%s)]", e.getStatusCode(), e.getReasonPhrase(), e.getMessage()));
		} catch (Exception e) {
			// Different handling; e.g. when returning null a nullpointer is thrown.
			System.out.println(e.getMessage());
		}
	}


	/**
	 * Check if the API-Key has update rights
	 * @param mps The {@link MaileonPingService} to use
	 */
	public static void checkUpdate(MaileonPingService mps) {
		System.out.print(" - Testing 'update': ");
		try {
			mps.checkUpdate();
			System.out.println("OK");
		} catch (MaileonException e) {
			System.out.println(String.format("FAILED [%s - %s (%s)]", e.getStatusCode(), e.getReasonPhrase(), e.getMessage()));
		} catch (Exception e) {
			// Different handling; e.g. when returning null a nullpointer is thrown.
			System.out.println(e.getMessage());
		}
	}


	/**
	 * Check if the API-Key has delete rights
	 * @param mps The {@link MaileonPingService} to use
	 */
	public static void checkDelete(MaileonPingService mps) {
		System.out.print(" - Testing 'delete': ");
		try {
			mps.checkDelete();
			System.out.println("OK");
		} catch (MaileonException e) {
			System.out.println(String.format("FAILED [%s - %s (%s)]", e.getStatusCode(), e.getReasonPhrase(), e.getMessage()));
		} catch (Exception e) {
			// Different handling; e.g. when returning null a nullpointer is thrown.
			System.out.println(e.getMessage());
		}
	}


}
