package com.maileon.api.reports;

import com.maileon.api.XmlUtils;
import com.maileon.api.contacts.ContactAdaptor;
import org.dom4j.Element;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

public class FactsAdaptor {

    public static List<Block> parseBlocks(Element xml) {
        List<Block> l = new ArrayList<>();
        for (Element e : (List<Element>) xml.elements("block")) {
            l.add(parseBlock(e));
        }
        return l;
    }


    public static List<Conversion> parseConversions(Element xml) {
        List<Conversion> l = new ArrayList<>();
        for (Element e : (List<Element>) xml.elements("conversion")) {
            l.add(parseConversion(e));
        }
        return l;
    }

    public static Conversion parseConversion(Element xml) {
        Conversion result = new Conversion();
        result.setTimestamp(XmlUtils.getNullableLongValue(xml.element("timestamp")));
        result.setTimestampSql(XmlUtils.getNullableStringValue(xml.element("timestamp_sql")));
        result.setContactId(XmlUtils.getNullableLongValue(xml.element("contact_id")));
        result.setContactEmail(XmlUtils.getNullableStringValue(xml.element("contact_email")));
        result.setValue(XmlUtils.getNullableDoubleValue(xml.element("value")));
        result.setMailingSentDate(XmlUtils.getNullableLongValue(xml.element("mailing_sent_date")));
        result.setMailingSentDateSql(XmlUtils.getNullableStringValue(xml.element("mailing_sent_date_sql")));
        result.setMailingId(XmlUtils.getNullableLongValue(xml.element("mailing_id")));
        result.setMailingName(XmlUtils.getNullableStringValue(xml.element("mailing_name")));
        result.setSiteId(XmlUtils.getNullableLongValue(xml.element("site_id")));
        result.setSiteName(XmlUtils.getNullableStringValue(xml.element("site_name")));
        result.setGoalId(XmlUtils.getNullableLongValue(xml.element("goal_id")));
        result.setGoalName(XmlUtils.getNullableStringValue(xml.element("goal_name")));
        result.setLinkId(XmlUtils.getNullableLongValue(xml.element("link_id")));
        result.setLinkUrl(XmlUtils.getNullableStringValue(xml.element("link_url")));
        return result;
    }

    public static List<UniqueConversion> parseUniqueConversions(Element xml) {
        List<UniqueConversion> l = new ArrayList<>();
        for (Element e : (List<Element>) xml.elements("unique_conversion")) {
            l.add(parseUniqueConversion(e));
        }
        return l;
    }

    public static UniqueConversion parseUniqueConversion(Element xml) {
        UniqueConversion result = new UniqueConversion();
        result.setContactId(XmlUtils.getNullableLongValue(xml.element("contact_id")));
        result.setContactEmail(XmlUtils.getNullableStringValue(xml.element("contact_email")));
        result.setContactExternalId(XmlUtils.getNullableStringValue(xml.element("contact_external_id")));
        result.setRevenue(XmlUtils.getNullableDoubleValue(xml.element("revenue")));
        result.setCountTotal(XmlUtils.getNullableLongValue(xml.element("count_total")));
        return result;
    }

    public static Block parseBlock(Element xml) {
        Block b = new Block();
        b.setTimestamp(parseTimestampChild(xml));
        b.setContact(parseContact(xml.element("contact")));
        b.setReason(XmlUtils.getNullableStringValue(xml.element("reason")));
        b.setOldStatus(XmlUtils.getNullableStringValue(xml.element("old_status")));
        b.setNewStatus(XmlUtils.getNullableStringValue(xml.element("new_status")));
        return b;
    }

    public static List<Bounce> parseBounces(Element xml) {
        List<Bounce> l = new ArrayList<>();
        for (Element e : (List<Element>) xml.elements("bounce")) {
            l.add(parseBounce(e));
        }
        return l;
    }

    public static Bounce parseBounce(Element xml) {
        Bounce b = new Bounce();
        b.setMailingId(parseMailingIdChild(xml));
        b.setTimestamp(parseTimestampChild(xml));
        b.setContact(parseContact(xml.element("contact")));
        b.setSource(XmlUtils.getNullableStringValue(xml.element("source")));
        b.setStatusCode(XmlUtils.getNullableStringValue(xml.element("status_code")));
        b.setType(XmlUtils.getNullableStringValue(xml.element("type")));
        return b;
    }

    public static List<Open> parseOpens(Element xml) {
        List<Open> l = new ArrayList<>();
        for (Element e : (List<Element>) xml.elements("open")) {
            l.add(parseOpen(e));
        }
        return l;
    }

    public static Open parseOpen(Element xml) {
        Open o = new Open();
        o.setContact(parseContact(xml.element("contact")));
        o.setDeviceType(parseDeviceTypeChild(xml));
        o.setEmailClient(parseEmailClient(xml.element("client")));
        o.setFormat(parseFormatChild(xml));
        o.setMailingId(parseMailingIdChild(xml));
        o.setTimestamp(parseTimestampChild(xml));
        return o;
    }

    public static List<Recipient> parseRecipients(Element xml) {
        List<Recipient> l = new ArrayList<>();
        for (Element e : (List<Element>) xml.elements("recipient")) {
            l.add(parseRecipient(e));
        }
        return l;
    }

    public static Recipient parseRecipient(Element xml) {
        Recipient r = new Recipient();
        r.setContact(parseContact(xml.element("contact")));
        r.setEmailClient(parseEmailClient(xml.element("client")));
        r.setMailingId(parseMailingIdChild(xml));
        r.setTimestamp(parseTimestampChild(xml));
        return r;
    }

    public static List<Click> parseClicks(Element xml) {
        List<Click> l = new ArrayList<>();
        for (Element e : (List<Element>) xml.elements("click")) {
            l.add(parseClick(e));
        }
        return l;
    }

    public static Click parseClick(Element xml) {
        Click c = new Click();
        c.setContact(parseContact(xml.element("contact")));
        c.setDeviceType(parseDeviceTypeChild(xml));
        c.setEmailClient(parseEmailClient(xml.element("client")));
        c.setFormat(parseFormatChild(xml));
        c.setLinkId(XmlUtils.getNullableLongValue(xml.element("link_id")));
        c.setLinkUrl(XmlUtils.getNullableStringValue(xml.element("link_url")));
        c.setMailingId(parseMailingIdChild(xml));
        c.setTimestamp(parseTimestampChild(xml));
        return c;
    }

    public static List<Unsubscriber> parseUnsubscriptions(Element xml) {
        List<Unsubscriber> l = new ArrayList<>();
        for (Element e : (List<Element>) xml.elements("unsubscription")) {
            l.add(parseUnsubscription(e));
        }
        return l;
    }

    public static Unsubscriber parseUnsubscription(Element xml) {
        Unsubscriber u = new Unsubscriber();
        u.setContact(parseContact(xml.element("contact")));
        u.setMailingId(parseMailingIdChild(xml));
        u.setSource(XmlUtils.getNullableStringValue(xml.element("source")));
        u.setTimestamp(parseTimestampChild(xml));
        return u;
    }

    public static List<Subscriber> parseSubscribers(Element xml) {
        List<Subscriber> subscribers = new ArrayList<>();
        for (Element e : (List<Element>) xml.elements("subscriber")) {
            subscribers.add(parseSubscriber(e));
        }
        return subscribers;
    }

    private static Subscriber parseSubscriber(Element xml) {
        Subscriber subscriber = new Subscriber();
        subscriber.setContact(parseContact(xml.element("contact")));
        subscriber.setMailingId(parseMailingIdChild(xml));
        subscriber.setTimestamp(parseTimestampChild(xml));
        return subscriber;
    }

    public static EmailClient parseEmailClient(Element xml) {
        if (xml == null) {
            return null;
        }
        EmailClient ec = new EmailClient();
        ec.setOsName(XmlUtils.getNullableStringValue(xml.element("os_name")));
        ec.setOsGroup(XmlUtils.getNullableStringValue(xml.element("os_group")));
        ec.setBrowser(XmlUtils.getNullableStringValue(xml.element("browser")));
        ec.setBrowserGroup(XmlUtils.getNullableStringValue(xml.element("browser_group")));
        ec.setBrowserType(XmlUtils.getNullableStringValue(xml.element("browser_type")));
        ec.setUserAgent(XmlUtils.getNullableStringValue(xml.element("user_agent")));
        ec.setRenderingEngine(XmlUtils.getNullableStringValue(xml.element("rendering_engine")));
        return ec;
    }

    public static ReportContact parseContact(Element xml) {
        return ContactAdaptor.parseReportContact(xml);
    }

    private static Long parseTimestampChild(Element xml) {
        Element ts = xml.element("timestamp");
        if (ts == null) {
            return null;
        }
        String s = XmlUtils.getNullableStringValue(ts);
        if (s == null) {
            return null;
        }
        return Timestamp.valueOf(s).getTime();
    }

    private static Long parseMailingIdChild(Element xml) {
        Element m = xml.element("mailing_id");
        if (m == null) {
            return null;
        }
        return XmlUtils.getNullableLongValue(m);
    }

    private static Format parseFormatChild(Element xml) {
        Element f = xml.element("format");
        if (f == null) {
            return null;
        }
        String s = XmlUtils.getNullableStringValue(f);
        if (s == null) {
            return null;
        }
        return s.equalsIgnoreCase("html") ? Format.HTML : Format.TEXT;
    }

    private static String parseDeviceTypeChild(Element xml) {
        Element dt = xml.element("device_type");
        if (dt == null) {
            return null;
        }
        return XmlUtils.getNullableStringValue(dt);
    }

}
